source("http://bioconductor.org/biocLite.R") #To download DESeq package (you can comment these lines out, they only need to be run once ever)
biocLite("factoextra")
# biocLite("DESeq2")

# source("http://bioconductor.org/biocLite.R")
# biocLite("genefilter")

setwd("C:/Users/ckenkel/Dropbox/AIMSpostdoc/PNGrnaseq")

library(DESeq) #To upload DESeq package - you need to do this every time you open R
library(gplots) # for venn diagram
library(ggplot2)
library(RColorBrewer)
library(affycoretools)
library(arrayQualityMetrics)
library(genefilter)
library(plotrix)
library(reshape2)
library(factoextra)

countsHost=read.table("allcounts_HostOnly.txt",header=TRUE,row.names=1) #Reading in the table of counts per isogroup by sample - you must open R in the same folder or change the working directory

countsZoox=read.table("allcounts_SymOnly.txt",header=FALSE,row.names=1) #Reading in the table of counts per isogroup by sample - you must open R in the same folder or change the working directory

head(countsHost) 
head(countsZoox)
names(countsZoox)<-names(countsHost)

length(countsHost[,1])  #45442 isogroups
length(countsZoox[,1]) #24076 isogroups

counts=counts[,order(names(counts))]
names(counts)



readsleft=c()
for (column in names(counts)) {
	val=sum(counts[,column])
	readsleft=append(readsleft,val)}

RLtable=data.frame(cbind(names(counts),readsleft))
RLtable$readsleft=as.numeric(as.character(RLtable$readsleft))
write.csv(RLtable,"readsleft.csv",quote=F) #min=425,459; max = 2,133,055

#Does %mapping differ for different Clade C sub-types at Dobu?
am2or3<-c(11.99,9.66,13.5,15.15,12.72)
am1<-c(18.9,17.51,16.36,11.57,14.63,14.82,15.81,17.10,16.19,14.41,14.90,16.28,13.31,11.08,15.42,12.72,12.09,10.25,16.22,12.55,14.96,16.19,13.79,13.95)

t.test(am1,am2or3,alternative="two.sided") 

#######################Creating table of conditions for your experiment, origin and transplant site

origin=treatment=c(1:length(names(counts)))
origin[grep("D",names(counts))]="Dobu"
origin[grep("I",names(counts))]="Illi"
treatment[grep("C",names(counts))]="control"
treatment[grep("B",names(counts))]="bubble"


conditions=data.frame(cbind(origin,treatment))
head(conditions)


############################################original DESeq methods

real=newCountDataSet(counts,conditions) 
real=estimateSizeFactors(real)

# ####all the data you ever wanted about quality control

cds=estimateDispersions(real,method="blind")
vsdBlind=varianceStabilizingTransformation(cds)

v="/Users/carlykenkel/Dropbox/AIMSpostdoc/PNGrnaseq/PooledHostSymAQMv2"

arrayQualityMetrics(vsdBlind,outdir=v,intgroup=c("treatment"),force=TRUE) #check .html output file in new folder

###############Remove outliers as detected; repeat arrayQualityMetrics above after regenerating newCountDataSet
###############to confirm that all outliers have been removed 

counts.nobad1=counts[,-c(22)] #According to heatmap AQMv1, Samples DC2 (still outlier for Dobu only set) adn IB5 (NOT outlier for Illi only set; none detected) are outliers for HOST, remove 
conditions.nobad1=conditions[-c(22),]

counts.nobad1=counts[,-c(14,32)] #According to heatmap AQMv1, Sample IB11 are outliers for SYM (still outlier for Illi Only set), remove ALSO must remove DB9 (contains clade D in addition to C1)
conditions.nobad1=conditions[-c(14,32),]


real=newCountDataSet(counts.nobad1,conditions.nobad1) 
real=estimateSizeFactors(real)

#############################

# #Real Analysis - USE good datasets - no outliers
library(DESeq)
real=newCountDataSet(counts[,c(29:30,32:58)],conditions[c(29:30,32:58),]) 
real=estimateSizeFactors(real)

real=estimateDispersions(real,method="pooled-CR",sharingMode="gene-est-only",modelFormula=count~origin+treatment)  
#can use gene-est-only, IF have >7 reps per treatment combo
# quartz()
# plotDispEsts(real)

vsd=getVarianceStabilizedData(real)
# write.csv(vsd, file="VSD_allsymgenes_nobadsam_Jan2015.csv", quote=F)

# ######################Determining quality filtering cutoffs

fit0=fitNbinomGLMs(real, count ~ 1) # null model: expression does not depend on anything
fit1=fitNbinomGLMs(real, count ~ treatment)
fit1a=fitNbinomGLMs(real, count ~ origin)
fit2=fitNbinomGLMs(real, count ~ treatment+origin)
fit3=fitNbinomGLMs(real, count ~ treatment+origin+treatment:origin)

pvals.i<-nbinomGLMTest(fit3,fit2) #testing significance of interaction term
pvals.t<-nbinomGLMTest(fit1,fit0) #testing significance of transplant term
pvals.o<-nbinomGLMTest(fit1a,fit0) #testing significance of origin term

pvalue=pvals.i #change to each type of pval: i, t and o
theta=seq(from=0,to=0.8,by=0.02)

filterChoices=data.frame(`mean`=rowMeans(counts(real)),`median`=apply((counts(real)),1,median),`min`=rowMin(counts(real)),`max`=rowMax(counts(real)),`sd`=rowSds(counts(real)))
rejChoices=sapply(filterChoices,function(f) filtered_R(alpha=0.1,filter=f,test=pvalue,theta=theta,method="BH"))
library("RColorBrewer")
myColours=brewer.pal(ncol(filterChoices),"Set1")

# #quartz()
# #windows()
matplot(theta,rejChoices,type="l",lty=1,col=myColours,lwd=2,xlab=expression(theta),ylab="number of rejections")
legend("bottomleft",legend=colnames(filterChoices),fill=myColours)

# #look for peak in graph - corresponds to correct theta and best-fit line for which metric to use - pick best theta for all tests
# #HOST: ori=0.7; treat = 0.5 to 0.6; inter = 0.6 => go with 0.6
# #SYMBIONT: ori=0.6; treat=0.6; inter=0.8 => go with 0.6

# #######################Quality Filtering Data based on theta - get rid of genes with low variance

# #FOR host
rs=rowSds(counts(real)) #using standard deviation as quality filtering metric based on analyses above
theta=0.6 #peak rej now occur at theta of 0.6 using SD for all..
use=(rs>quantile(rs,probs=theta)) ###
table(use) 
# # use
# # use
# # FALSE  TRUE 
# # 27265 18177  #have 18k genes that pass filter - for HOST
# # use
# # FALSE  TRUE 
# # 14447  9629  #for SYM


#####RERUN within pop ONLY
#Dobu, treatment model only, theta =0.6 still
# use
# FALSE  TRUE 
# 27267 18175 

#####RERUN within pop ONLY
#Illi, treatment model only, theta =0.5 ALSO no outliers
# use
#FALSE  TRUE 
#22723 22719  

#####RERUN within pop ONLY SYMBIONT
#Dobu, treatment model only, theta =0.7 now
# use
# FALSE  TRUE 
# 16853  7223 

#####RERUN within pop ONLY SYMBIONT
#Illi, treatment model only, theta =0.7 now
# use
#FALSE  TRUE 
#16853  7223  



realFilt=real[use,]
vsd=getVarianceStabilizedData(realFilt)

######################## Now for the real Model Testing

fit0=fitNbinomGLMs(realFilt, count ~ 1) # null model: expression does not depend on anything
fit1=fitNbinomGLMs(realFilt, count ~ treatment)
fit1a=fitNbinomGLMs(realFilt, count ~ origin)
fit2=fitNbinomGLMs(realFilt, count ~ treatment+origin)
fit3=fitNbinomGLMs(realFilt, count ~ treatment+origin+treatment:origin)


# testing section
pvals.i<-nbinomGLMTest(fit3,fit2) #testing significance of interaction term
pvals.t<-nbinomGLMTest(fit1,fit0) #testing significance of treatment term
pvals.o<-nbinomGLMTest(fit1a,fit0) #testing significance of origin term


#making non-convergent model p-values NA's
pvals.i=data.frame(pvals.i)
rownames(fit3)->rownames(pvals.i)
badmods=subset(fit3,(!fit3[,6]))
for ( i in rownames(badmods)){pvals.i[i,1]<-NA}

pvals.t=data.frame(pvals.t)
rownames(fit1)->rownames(pvals.t)
badmods=subset(fit1,(!fit1[,4]))
for ( i in rownames(badmods)){pvals.t[i,1]<-NA}

pvals.o=data.frame(pvals.o)
rownames(fit1a)->rownames(pvals.o)
badmods=subset(fit1a,(!fit1a[,4]))
for ( i in rownames(badmods)){pvals.o[i,1]<-NA}


summary(pvals.i)
summary(pvals.t)
summary(pvals.o)


#multiple test correction - adjust p-values using Benjamini-Hochburg

adjp.i<-p.adjust(pvals.i$pvals.i,method="BH")
adjp.t<-p.adjust(pvals.t$pvals.t,method="BH")
adjp.o<-p.adjust(pvals.o$pvals.o,method="BH")


PooledPvals<-cbind(vsd, "adjp.t" = adjp.t, "adjp.o" = adjp.o, "adjp.i" = adjp.i) #creating table of all multiple test corrected p-values with variance stabilized count data 

#toss in raw pvalues as well
do<-cbind(PooledPvals, "pval.t" = pvals.t$pvals.t, "pval.o" = pvals.o$pvals.o, "pval.i" = pvals.i$pvals.i) 

write.csv(do, file="VSDandPVALSnobadsamSym_SDtheta06_feb12.csv", quote=F) #writing an output file of vsd plus p-values


########################Permutation tests...how many significant Seep genes if Host dataset subsampled to symbiodinium size?

d=read.csv("VSDandPVALSnobadsamHost_SDtheta06_jan21_plusMicrobe_plusClusterID.csv") #VSDandPVALSnobadsamHost_SDtheta06_jan21_plusMicrobe_plusClusterID #VSDandPVALSnobadsamSym_SDtheta06_feb12.csv
#Reruns, within pop: VSDandPVALSnobadsamHostDobuOnly_SDtheta06_oct13.csv
#VSDandPVALSnobadsamSymIlliOnly_SDtheta07_oct13.csv

rownames(d)<-d$X
head(d)

d2=read.csv("VSDandPVALSnobadsamSym_SDtheta06_feb12.csv")
rownames(d2)<-d2$X
# Observed difference
diff.observed = 986 #number obs DEIs sym dataset
 
number_of_permutations = 1000
     
diff.random = NULL
for (i in 1 : number_of_permutations) {
 
    # Sample from the Host "good" dataset without replacement to match size of symbiont dataset
    pick = sample (rownames(d2), 9629)
    shuffled=as.data.frame(d2[pick,])
    
	# Calculate FDR-corrected p-values for smaller random subset 
	sig<-as.data.frame(p.adjust(shuffled$pval.o,method="BH"))
	names(sig)<-"adjp.o"
	b.random=length(sig[sig$adjp.o<=0.1 & !is.na(sig$adjp.o),])
 
    # Null (permuated) difference
    diff.random[i] = b.random #number of DEIs for artifically small host dataset
}
 
# P-value is the fraction of how many times the permuted difference is equal or more extreme than the observed difference
 
pvalue = sum(abs(diff.random) >= abs(diff.observed)) / number_of_permutations
mean(diff.random)
print (pvalue)
#p<0.0001***

###########Between Groups analysis

library(made4)
 
SiteHost=c(rep("Seep",times=14),rep("Ctrl",times=14),rep("Seep",times=14),rep("Ctrl",times=13))
SiteSym=c(rep("Seep",times=13),rep("Ctrl",times=15),rep("Seep",times=14),rep("Ctrl",times=15))

tdat<-(d2[,2:58]) #d2[,2:58] sym; d[,2:56] host

pca<-bga(tdat,SiteSym,type="pca")
plot.bga(pca)
topgenes(pca,n=5)

#NOW, pick out only TOP DEGs
topnum=50 # number of top candidates per comparison. The total number over all comparisons must not exceed the number of samples

GEs<-d2 #pick either host or sym dataset

ori=head(GEs[order(GEs$adjp.o),],topnum)
treat=head(GEs[order(GEs$adjp.t),],topnum)
sig=data.frame(rbind(ori[!(ori$X %in% treat$X),],treat))
length(sig[,1]) #Top1000:1767 


#Transpose expression values only
tsig=t(treat[,2:58]) #3:57 host; 2:58 sym
tsig[1:5,1:5]


# apply PCA - scale. = TRUE is highly 
# advisable, but default is FALSE.
library(devtools)

library(ggbiplot)
 
pca <- prcomp(tsig,
      center = TRUE,
      scale. = TRUE) 
summary(pca)
str(pca)

scores=pca$x

scores[,1:2]

plot(scores[,1], scores[,2], xlab="PCA 1", ylab="PCA 2",type="n")
Ds=grep("DB",row.names(scores))
Dc=grep("DC",row.names(scores))
Is=grep("IB",row.names(scores))
Ic=grep("IC",row.names(scores))
points(scores[1,1],scores[1,2],pch=50,col="red")
points(scores[c(2:5),1],scores[c(2:5),2],pch=49,col="red")
points(scores[6,1],scores[6,2],pch=78,col="red")
points(scores[c(7:8),1],scores[c(7:8),2],pch=49,col="red")
points(scores[9,1],scores[9,2],pch=51,col="red")
points(scores[c(10:11),1],scores[c(10:11),2],pch=49,col="red")
points(scores[12,1],scores[12,2],pch=51,col="red")
points(scores[13,1],scores[13,2],pch=49,col="red")
points(scores[20,1],scores[20,2],pch=50,col="blue")
points(scores[c(21:27),1],scores[c(21:27),2],pch=49,col="blue")
points(scores[28,1],scores[28,2],pch=50,col="blue")
points(scores[Is,1],scores[Is,2],pch=49,col="red")
points(scores[Ic,1],scores[Ic,2],pch=49,col="blue")


SiteHostSeep=c(rep("Dobu-Seep",times=14),rep("Dobu-Ctrl",times=14),rep("Illi-Seep",times=14),rep("Illi-Ctrl",times=13))
SiteSymSeep=c(rep("Dobu-Seep",times=13),rep("Dobu-Ctrl",times=15),rep("Illi-Seep",times=14),rep("Illi-Ctrl",times=15))

g <- ggbiplot(pca, #obs.scale = 1, #var.scale = 1, 
              groups = SiteSymSeep, ellipse = TRUE,     #also did by TIME
              circle = TRUE,var.axes=FALSE,scale=1)
g <- g + scale_color_discrete(name = '')
g <- g + theme_bw()
g <- g + theme(legend.direction = 'horizontal', 
              legend.position = 'top')

print(g)
 
#For sym: top and bottom 5 loadings for PC2
pca$rotation[order(pca$rotation[,2]),2]

#c_sym_59999       c_sym_28988       c_sym_22292       c_sym_29386       c_sym_71409
#-0.276372200      -0.191813194      -0.190377236      -0.186078833      -0.159680393 
#c_sym_40040       c_sym_31320       c_sym_11928       c_sym_23149       c_sym_92871
#0.218015689       0.220430388       0.227161075       0.313072843       0.320725043

#########Generating directional GO output for DESeq results

#transplant
fit1$direction=ifelse(fit1$treatmentcontrol<0,1,-1) #red=higher expression in bubble
fit1$pval<-(-log((pvals.t$pvals.t+0.0000000001),10))*(fit1$direction)

#origin
fit1a$direction=ifelse(fit1a$originIlli>0,1,-1) #red=higher expression in Illi
fit1a$pval<-(-log((pvals.o$pvals.o+0.0000000001),10))*(fit1a$direction)

#interaction
fit3$direction=ifelse(fit3[,4]<0,1,-1) #red=higher expression in Dobi:Ctrl; Illi:bubble / blue=higher exp in Illi: Ctrl; Dobu: Bubble
fit3$pval<-(-log((pvals.i$pvals.i+0.0000000001),10))*(fit3$direction)


interaction<-cbind("gene"=rownames(fit3),"pval"=fit3$pval)
treatment<-cbind("gene"=rownames(fit1),"pval"=fit1$pval) 
origin<-cbind("gene"=rownames(fit1a),"pval"=fit1a$pval) 


write.csv(interaction,file="GOinteractionJan21sym.csv",quote=F,row.names=F)
write.csv(treatment,file="GOtreatmentJan21sym.csv",quote=F,row.names=F)
write.csv(origin,file="GOoriginJan21sym.csv",quote=F,row.names=F)


########################## counting, venn diagram:

p<-data.frame(d)
inter=row.names(p[p$adjp.i<=0.1 & !is.na(p$adjp.i),])
ori=row.names(p[p$adjp.o<=0.1 & !is.na(p$adjp.o),])
treat=row.names(p[p$adjp.t<=0.1 & !is.na(p$adjp.t),])
microbe=row.names(p[p$adjp.M<=0.05 & !is.na(p$adjp.M),])
#t.o=row.names(p[p$adjp.t.o<=0.05 & !is.na(p$adjp.t.o),])

candidates=list("DobuSeep,P=0.1"=treatD,"IlliSeep,P=0.1"=treatI)
candidates5=list("seep"=treat,"origin"=ori,"interaction"=inter)
library(gplots)
quartz()
venn(candidates)

############Overlap between datasets
treatD=row.names(d2[d2$adjp.t<=0.1 & !is.na(d2$adjp.t),])
treatI=(d[d$adjp.t<=0.1 & !is.na(d$adjp.t),])

is=row.names(d[d$adjp.t<=0.1 & !is.na(d$adjp.t),])

treatI[treatI %in% treatD]


##########################################Categorical GO files for gomwu scripts

d=read.csv("VSDandPVALSnobadsamSym_SDtheta06_feb12.csv") #VSDandPVALSnobadsamSym_SDtheta06_feb12.csv VSDandPVALSnobadsamHost_SDtheta06_jan21
rownames(d)<-d$X

d$binary=ifelse(d$adjp.t<=0.1 & !is.na(d$adjp.t),1,0)
d$binary=as.factor(d$binary)
summary(d)
names(d)
GObinary<-d[,c(1,69)]
write.csv(GObinary,file="GObinaryTreatsym.csv",quote=F,row.names=F)

d$binary2=ifelse(d$adjp.o<=0.05 & !is.na(d$adjp.o) & d$adjp.i>0.05 & d$adjp.t>0.05,1,0)
d$binary2=as.factor(d$binary2)
summary(d)
names(d)
GObinary<-d[,c(1,68)]
write.csv(GObinary,file="GObinaryOrisym.csv",quote=F,row.names=F)


goT=read.csv("GOtreatmentJan21sym.csv") #Jan21 = HOST; add sym at the end for sym files
goO=read.csv("GOoriginJan21sym.csv")

d$directionT<-goT$pval
d$directionO<-goO$pval

gg=read.table("SymC_denovo_iso2gene.tab",header=F,sep="	",quote="",row.names=NULL,stringsAsFactors=FALSE) #amil_iso2gene.tab SymC_denovo_iso2gene.tab

treat=d[d$adjp.t<=0.1 & !is.na(d$adjp.t) & d$adjp.o>0.1 & d$adjp.i>0.1,] #same genes used in Venn, now to assess direction of change and heatmap plot w/gene names
nrow(treat)
treatup=treat[treat$directionT>0 & !is.na(treat$directionT),] #genes up-regulated in bubble = 15/26; 65/96 sym
treatdown=treat[treat$directionT<0 & !is.na(treat$directionT),] #genes down-regulated in bubble = 13/27; 11/32 sym

ori=d[d$adjp.o<=0.05 & !is.na(d$adjp.o) & d$adjp.t>0.05 & d$adjp.i>0.05,] #same genes used in Venn, now to assess direction of change and heatmap plot w/gene names

oriup=ori[ori$directionO>0,] #genes up-regulated in Illi = 101; 373 sym
oridown=ori[ori$directionO<0,] #genes up-reg in Dobu = 213; 346 sym


inter=d[d$adjp.i<=0.1 & !is.na(d$adjp.i),]
interup=inter[inter$directionI>0,] #genes up-regulated in Illi
interdown=inter[inter$directionI<0,] 


is=rownames(treatdown) #change to list of interest each time
dat=as.data.frame(treatdown)

edata=c(2:58) #only columns with your expression data - NOTE 2:58 for zoox; 2:56 for host

is=treatD
dat<-d2

is=c("c_sym_59999", "c_sym_28988", "c_sym_22292", "c_sym_29386" ,"c_sym_71409","c_sym_40040", "c_sym_31320", "c_sym_11928", "c_sym_23149", "c_sym_92871")

#########################append gene names to isogroups
sel=c();gnms=c()
for ( i in is){
	if (i %in% dat$X){
		sel=rbind(sel,dat[dat$X==i,])
		gnms=append(gnms,substr(paste(as.character(gg$V2[gg$V1==i]),collapse="."),1,50))
	}
}
row.names(sel)=paste(gnms,sel$X,sep=".")

exp=sel[,edata]
if (length(exp[,1])==1) { exp=rbind(exp,exp) }
nrow(exp) #should match number of genes in list of interest above

write.csv(exp,file="HostOplusT.csv",quote=FALSE)


means=apply(exp,1,mean) # means of rows
expc=exp-means #rescale expression data so it's up and down

#greenred=colorRampPalette(c("purple","blue","black","orange","yellow"),bias=1)(n=150) #lower bias number gives more greens; higher bias gives more reds
#par(mfrow=c(2,1), mar=c(0.3, 5.5, 3, 2))
#library(RColorBrewer)
col=color = colorRampPalette(rev(brewer.pal(n = 7, name ="RdBu")),bias=2.25)(30)
# quartz()
# pheatmap(expc,color=col,cluster_cols=F,clustering_distance_rows="correlation") 
quartz()
pdf("SymHeatmapTreatup.pdf",width=7,height=18)
pheatmap(expc,color=col,cluster_cols=F,clustering_distance_rows="correlation") #plot the heatmap
dev.off()

